@@ -23,16 +23,16 @@ fn main() {
     let man = load_image::<f32>(format!("{}/man.jpg", assets_dir.display()), false);
     let hst = histogram(&man, 256, 0.0, 255.0);
 
-    let disp_img = div(&man, &constant(255 as f32, man.dims()), false);
+    let disp_img = div(&man, &constant(255_f32, man.dims()), false);
 
     loop {
         img_wnd.draw_image(&disp_img, None);
         hst_wnd.draw_hist(&hst, 0.0, 255.0, None);
 
-        if img_wnd.is_closed() == true {
+        if img_wnd.is_closed() {
             break;
         }
-        if hst_wnd.is_closed() == true {
+        if hst_wnd.is_closed() {
             break;
         }
     }
