@@ -1,30 +1,48 @@
-extern crate libc;
+use super::core::{
+    af_array, AfError, Array, FloatingPoint, HasAfEnum, MatProp, NormType, HANDLE_ERROR,
+};
 
-use self::libc::{c_double, c_int, c_uint};
-use crate::array::Array;
-use crate::defines::{AfError, MatProp, NormType};
-use crate::error::HANDLE_ERROR;
-use crate::util::{to_u32, AfArray, MutAfArray, MutDouble};
-use crate::util::{FloatingPoint, HasAfEnum};
+use libc::{c_double, c_int, c_uint};
 
-#[allow(dead_code)]
 extern "C" {
-    fn af_svd(u: MutAfArray, s: MutAfArray, vt: MutAfArray, input: AfArray) -> c_int;
-    fn af_svd_inplace(u: MutAfArray, s: MutAfArray, vt: MutAfArray, input: AfArray) -> c_int;
-    fn af_lu(lower: MutAfArray, upper: MutAfArray, pivot: MutAfArray, input: AfArray) -> c_int;
-    fn af_lu_inplace(pivot: MutAfArray, input: AfArray, is_lapack_piv: c_int) -> c_int;
-    fn af_qr(q: MutAfArray, r: MutAfArray, tau: MutAfArray, input: AfArray) -> c_int;
-    fn af_qr_inplace(tau: MutAfArray, input: AfArray) -> c_int;
-    fn af_cholesky(out: MutAfArray, info: *mut c_int, input: AfArray, is_upper: c_int) -> c_int;
-    fn af_cholesky_inplace(info: *mut c_int, input: AfArray, is_upper: c_int) -> c_int;
-    fn af_solve(x: MutAfArray, a: AfArray, b: AfArray, options: c_uint) -> c_int;
-    fn af_solve_lu(x: MutAfArray, a: AfArray, piv: AfArray, b: AfArray, options: c_uint) -> c_int;
-    fn af_inverse(out: MutAfArray, input: AfArray, options: c_uint) -> c_int;
-    fn af_rank(rank: *mut c_uint, input: AfArray, tol: c_double) -> c_int;
-    fn af_det(det_real: MutDouble, det_imag: MutDouble, input: AfArray) -> c_int;
-    fn af_norm(out: MutDouble, input: AfArray, ntype: c_uint, p: c_double, q: c_double) -> c_int;
-    fn af_is_lapack_available(out: *mut c_int) -> c_int;
-    fn af_pinverse(out: MutAfArray, input: AfArray, tol: c_double, options: c_int) -> c_int;
+    fn af_svd(u: *mut af_array, s: *mut af_array, vt: *mut af_array, input: af_array) -> c_int;
+    fn af_svd_inplace(
+        u: *mut af_array,
+        s: *mut af_array,
+        vt: *mut af_array,
+        input: af_array,
+    ) -> c_int;
+    fn af_lu(
+        lower: *mut af_array,
+        upper: *mut af_array,
+        pivot: *mut af_array,
+        input: af_array,
+    ) -> c_int;
+    fn af_lu_inplace(pivot: *mut af_array, input: af_array, is_lapack_piv: bool) -> c_int;
+    fn af_qr(q: *mut af_array, r: *mut af_array, tau: *mut af_array, input: af_array) -> c_int;
+    fn af_qr_inplace(tau: *mut af_array, input: af_array) -> c_int;
+    fn af_cholesky(out: *mut af_array, info: *mut c_int, input: af_array, is_upper: bool) -> c_int;
+    fn af_cholesky_inplace(info: *mut c_int, input: af_array, is_upper: bool) -> c_int;
+    fn af_solve(x: *mut af_array, a: af_array, b: af_array, options: c_uint) -> c_int;
+    fn af_solve_lu(
+        x: *mut af_array,
+        a: af_array,
+        piv: af_array,
+        b: af_array,
+        options: c_uint,
+    ) -> c_int;
+    fn af_inverse(out: *mut af_array, input: af_array, options: c_uint) -> c_int;
+    fn af_rank(rank: *mut c_uint, input: af_array, tol: c_double) -> c_int;
+    fn af_det(det_real: *mut c_double, det_imag: *mut c_double, input: af_array) -> c_int;
+    fn af_norm(
+        out: *mut c_double,
+        input: af_array,
+        ntype: c_uint,
+        p: c_double,
+        q: c_double,
+    ) -> c_int;
+    fn af_is_lapack_available(out: *mut bool) -> c_int;
+    fn af_pinverse(out: *mut af_array, input: af_array, tol: c_double, options: c_uint) -> c_int;
 }
 
 /// Perform Singular Value Decomposition
@@ -50,25 +68,24 @@ extern "C" {
 /// The second Array is the output array containing the diagonal values of sigma, (singular values of the input matrix))
 ///
 /// The third Array is the output array containing V ^ H
-#[allow(unused_mut)]
 pub fn svd<T>(input: &Array<T>) -> (Array<T>, Array<T::BaseType>, Array<T>)
 where
     T: HasAfEnum + FloatingPoint,
     T::BaseType: HasAfEnum,
 {
-    let mut u: i64 = 0;
-    let mut s: i64 = 0;
-    let mut vt: i64 = 0;
     unsafe {
+        let mut u: af_array = std::ptr::null_mut();
+        let mut s: af_array = std::ptr::null_mut();
+        let mut vt: af_array = std::ptr::null_mut();
         let err_val = af_svd(
-            &mut u as MutAfArray,
-            &mut s as MutAfArray,
-            &mut vt as MutAfArray,
-            input.get() as AfArray,
+            &mut u as *mut af_array,
+            &mut s as *mut af_array,
+            &mut vt as *mut af_array,
+            input.get(),
         );
         HANDLE_ERROR(AfError::from(err_val));
+        (u.into(), s.into(), vt.into())
     }
-    (u.into(), s.into(), vt.into())
 }
 
 /// Perform Singular Value Decomposition inplace
@@ -95,25 +112,24 @@ where
 /// The second Array is the output array containing the diagonal values of sigma, (singular values of the input matrix))
 ///
 /// The third Array is the output array containing V ^ H
-#[allow(unused_mut)]
 pub fn svd_inplace<T>(input: &mut Array<T>) -> (Array<T>, Array<T::BaseType>, Array<T>)
 where
     T: HasAfEnum + FloatingPoint,
     T::BaseType: HasAfEnum,
 {
-    let mut u: i64 = 0;
-    let mut s: i64 = 0;
-    let mut vt: i64 = 0;
     unsafe {
+        let mut u: af_array = std::ptr::null_mut();
+        let mut s: af_array = std::ptr::null_mut();
+        let mut vt: af_array = std::ptr::null_mut();
         let err_val = af_svd_inplace(
-            &mut u as MutAfArray,
-            &mut s as MutAfArray,
-            &mut vt as MutAfArray,
-            input.get() as AfArray,
+            &mut u as *mut af_array,
+            &mut s as *mut af_array,
+            &mut vt as *mut af_array,
+            input.get(),
         );
         HANDLE_ERROR(AfError::from(err_val));
+        (u.into(), s.into(), vt.into())
     }
-    (u.into(), s.into(), vt.into())
 }
 
 /// Perform LU decomposition
@@ -131,24 +147,23 @@ where
 /// The second Array will contain the lower triangular matrix of the LU decomposition.
 ///
 /// The third Array will contain the permutation indices to map the input to the decomposition.
-#[allow(unused_mut)]
 pub fn lu<T>(input: &Array<T>) -> (Array<T>, Array<T>, Array<i32>)
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut lower: i64 = 0;
-    let mut upper: i64 = 0;
-    let mut pivot: i64 = 0;
     unsafe {
+        let mut lower: af_array = std::ptr::null_mut();
+        let mut upper: af_array = std::ptr::null_mut();
+        let mut pivot: af_array = std::ptr::null_mut();
         let err_val = af_lu(
-            &mut lower as MutAfArray,
-            &mut upper as MutAfArray,
-            &mut pivot as MutAfArray,
-            input.get() as AfArray,
+            &mut lower as *mut af_array,
+            &mut upper as *mut af_array,
+            &mut pivot as *mut af_array,
+            input.get(),
         );
         HANDLE_ERROR(AfError::from(err_val));
+        (lower.into(), upper.into(), pivot.into())
     }
-    (lower.into(), upper.into(), pivot.into())
 }
 
 /// Perform inplace LU decomposition
@@ -162,21 +177,16 @@ where
 ///
 /// An Array with permutation indices to map the input to the decomposition. Since, the input
 /// matrix is modified in place, only pivot values are returned.
-#[allow(unused_mut)]
 pub fn lu_inplace<T>(input: &mut Array<T>, is_lapack_piv: bool) -> Array<i32>
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut pivot: i64 = 0;
     unsafe {
-        let err_val = af_lu_inplace(
-            &mut pivot as MutAfArray,
-            input.get() as AfArray,
-            is_lapack_piv as c_int,
-        );
+        let mut pivot: af_array = std::ptr::null_mut();
+        let err_val = af_lu_inplace(&mut pivot as *mut af_array, input.get(), is_lapack_piv);
         HANDLE_ERROR(AfError::from(err_val));
+        pivot.into()
     }
-    pivot.into()
 }
 
 /// Perform QR decomposition
@@ -195,24 +205,23 @@ where
 ///
 /// The third Array will contain additional information needed for solving a least squares problem
 /// using q and r
-#[allow(unused_mut)]
 pub fn qr<T>(input: &Array<T>) -> (Array<T>, Array<T>, Array<T>)
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut q: i64 = 0;
-    let mut r: i64 = 0;
-    let mut tau: i64 = 0;
     unsafe {
+        let mut q: af_array = std::ptr::null_mut();
+        let mut r: af_array = std::ptr::null_mut();
+        let mut tau: af_array = std::ptr::null_mut();
         let err_val = af_qr(
-            &mut q as MutAfArray,
-            &mut r as MutAfArray,
-            &mut tau as MutAfArray,
-            input.get() as AfArray,
+            &mut q as *mut af_array,
+            &mut r as *mut af_array,
+            &mut tau as *mut af_array,
+            input.get(),
         );
         HANDLE_ERROR(AfError::from(err_val));
+        (q.into(), r.into(), tau.into())
     }
-    (q.into(), r.into(), tau.into())
 }
 
 /// Perform inplace QR decomposition
@@ -224,17 +233,16 @@ where
 /// # Return Values
 ///
 /// An Array with additional information needed for unpacking the data.
-#[allow(unused_mut)]
 pub fn qr_inplace<T>(input: &mut Array<T>) -> Array<T>
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut tau: i64 = 0;
     unsafe {
-        let err_val = af_qr_inplace(&mut tau as MutAfArray, input.get() as AfArray);
+        let mut tau: af_array = std::ptr::null_mut();
+        let err_val = af_qr_inplace(&mut tau as *mut af_array, input.get());
         HANDLE_ERROR(AfError::from(err_val));
+        tau.into()
     }
-    tau.into()
 }
 
 /// Perform Cholesky decomposition
@@ -252,23 +260,22 @@ where
 ///
 /// If the integer is 0, it means the cholesky decomposition passed. Otherwise, it will contain the rank at
 /// which the decomposition failed.
-#[allow(unused_mut)]
 pub fn cholesky<T>(input: &Array<T>, is_upper: bool) -> (Array<T>, i32)
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut temp: i64 = 0;
-    let mut info: i32 = 0;
     unsafe {
+        let mut temp: af_array = std::ptr::null_mut();
+        let mut info: i32 = 0;
         let err_val = af_cholesky(
-            &mut temp as MutAfArray,
+            &mut temp as *mut af_array,
             &mut info as *mut c_int,
-            input.get() as AfArray,
-            is_upper as c_int,
+            input.get(),
+            is_upper,
         );
         HANDLE_ERROR(AfError::from(err_val));
+        (temp.into(), info)
     }
-    (temp.into(), info)
 }
 
 /// Perform inplace Cholesky decomposition
@@ -282,18 +289,13 @@ where
 ///
 /// A signed 32-bit integer. If the integer is 0, it means the cholesky decomposition passed. Otherwise,
 /// it will contain the rank at which the decomposition failed.
-#[allow(unused_mut)]
 pub fn cholesky_inplace<T>(input: &mut Array<T>, is_upper: bool) -> i32
 where
     T: HasAfEnum + FloatingPoint,
 {
     let mut info: i32 = 0;
     unsafe {
-        let err_val = af_cholesky_inplace(
-            &mut info as *mut c_int,
-            input.get() as AfArray,
-            is_upper as c_int,
-        );
+        let err_val = af_cholesky_inplace(&mut info as *mut c_int, input.get(), is_upper);
         HANDLE_ERROR(AfError::from(err_val));
     }
     info
@@ -312,22 +314,21 @@ where
 /// # Return Values
 ///
 /// An Array which is the matrix of unknown variables
-#[allow(unused_mut)]
 pub fn solve<T>(a: &Array<T>, b: &Array<T>, options: MatProp) -> Array<T>
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut temp: i64 = 0;
     unsafe {
+        let mut temp: af_array = std::ptr::null_mut();
         let err_val = af_solve(
-            &mut temp as MutAfArray,
-            a.get() as AfArray,
-            b.get() as AfArray,
-            to_u32(options) as c_uint,
+            &mut temp as *mut af_array,
+            a.get(),
+            b.get(),
+            options as c_uint,
         );
         HANDLE_ERROR(AfError::from(err_val));
+        temp.into()
     }
-    temp.into()
 }
 
 /// Solve a system of equations
@@ -344,23 +345,22 @@ where
 /// # Return Values
 ///
 /// An Array which is the matrix of unknown variables
-#[allow(unused_mut)]
 pub fn solve_lu<T>(a: &Array<T>, piv: &Array<i32>, b: &Array<T>, options: MatProp) -> Array<T>
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut temp: i64 = 0;
     unsafe {
+        let mut temp: af_array = std::ptr::null_mut();
         let err_val = af_solve_lu(
-            &mut temp as MutAfArray,
-            a.get() as AfArray,
-            piv.get() as AfArray,
-            b.get() as AfArray,
-            to_u32(options) as c_uint,
+            &mut temp as *mut af_array,
+            a.get(),
+            piv.get(),
+            b.get(),
+            options as c_uint,
         );
         HANDLE_ERROR(AfError::from(err_val));
+        temp.into()
     }
-    temp.into()
 }
 
 /// Compute inverse of a matrix
@@ -375,21 +375,16 @@ where
 /// # Return Values
 ///
 /// An Array with values of the inverse of input matrix.
-#[allow(unused_mut)]
 pub fn inverse<T>(input: &Array<T>, options: MatProp) -> Array<T>
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut temp: i64 = 0;
     unsafe {
-        let err_val = af_inverse(
-            &mut temp as MutAfArray,
-            input.get() as AfArray,
-            to_u32(options) as c_uint,
-        );
+        let mut temp: af_array = std::ptr::null_mut();
+        let err_val = af_inverse(&mut temp as *mut af_array, input.get(), options as c_uint);
         HANDLE_ERROR(AfError::from(err_val));
+        temp.into()
     }
-    temp.into()
 }
 
 /// Find rank of a matrix
@@ -402,18 +397,13 @@ where
 /// # Return Values
 ///
 /// An unsigned 32-bit integer which is the rank of the input matrix.
-#[allow(unused_mut)]
 pub fn rank<T>(input: &Array<T>, tol: f64) -> u32
 where
     T: HasAfEnum + FloatingPoint,
 {
     let mut temp: u32 = 0;
     unsafe {
-        let err_val = af_rank(
-            &mut temp as *mut c_uint,
-            input.get() as AfArray,
-            tol as c_double,
-        );
+        let err_val = af_rank(&mut temp as *mut c_uint, input.get(), tol);
         HANDLE_ERROR(AfError::from(err_val));
     }
     temp
@@ -430,7 +420,6 @@ where
 /// A tuple of 32-bit floating point values.
 ///
 /// If the input matrix is non-complex type, only first values of tuple contains the result.
-#[allow(unused_mut)]
 pub fn det<T>(input: &Array<T>) -> (f64, f64)
 where
     T: HasAfEnum + FloatingPoint,
@@ -439,9 +428,9 @@ where
     let mut imag: f64 = 0.0;
     unsafe {
         let err_val = af_det(
-            &mut real as MutDouble,
-            &mut imag as MutDouble,
-            input.get() as AfArray,
+            &mut real as *mut c_double,
+            &mut imag as *mut c_double,
+            input.get(),
         );
         HANDLE_ERROR(AfError::from(err_val));
     }
@@ -462,7 +451,6 @@ where
 /// # Return Values
 ///
 /// A 64-bit floating point value that contains the norm of input matrix.
-#[allow(unused_mut)]
 pub fn norm<T>(input: &Array<T>, ntype: NormType, p: f64, q: f64) -> f64
 where
     T: HasAfEnum + FloatingPoint,
@@ -470,11 +458,11 @@ where
     let mut out: f64 = 0.0;
     unsafe {
         let err_val = af_norm(
-            &mut out as MutDouble,
-            input.get() as AfArray,
+            &mut out as *mut c_double,
+            input.get(),
             ntype as c_uint,
-            p as c_double,
-            q as c_double,
+            p,
+            q,
         );
         HANDLE_ERROR(AfError::from(err_val));
     }
@@ -491,11 +479,11 @@ where
 ///
 /// Return a boolean indicating if ArrayFire was compiled with lapack support
 pub fn is_lapack_available() -> bool {
-    let mut temp: i32 = 0;
+    let mut temp: bool = false;
     unsafe {
-        af_is_lapack_available(&mut temp as *mut c_int);
+        af_is_lapack_available(&mut temp as *mut bool);
     }
-    temp > 0 // Return boolean fla
+    temp
 }
 
 /// Psuedo Inverse of Matrix
@@ -519,15 +507,15 @@ pub fn pinverse<T>(input: &Array<T>, tolerance: f64, option: MatProp) -> Array<T
 where
     T: HasAfEnum + FloatingPoint,
 {
-    let mut out: i64 = 0;
     unsafe {
+        let mut out: af_array = std::ptr::null_mut();
         let err_val = af_pinverse(
-            &mut out as MutAfArray,
-            input.get() as AfArray,
+            &mut out as *mut af_array,
+            input.get(),
             tolerance,
-            option as c_int,
+            option as c_uint,
         );
         HANDLE_ERROR(AfError::from(err_val));
+        out.into()
     }
-    out.into()
 }
